---- More about Arrays ----
Creating Arrays:

// fill an Array with 10 random numbers between 0 and 4:
a = Array.fill( 10, { 5.rand } );

// note the difference with:
a = Array.fill( 10, 5.rand );

// the latter fills the array with the result of doing one time 5.rand, whereas the first fills the array by executing the function { 5.rand } for each item, thus creating different random numbers for each item. If you want the array to be filled with different items, you have to use the function method. Another example:

a = Array.fill( 10, [1,2,3,4] );
a[0]
[3] = 5;
// all sub arrays are changed:
a

b = Array.fill( 10, { [1,2,3,4] } );
b[0][3] = 5;
// only the first has changed:
b

// you can also add an argument to the fill function:
a = Array.fill( 10, { |i| i * 100 + 20 } );
a = Array.fill( 10, { arg i; i * 100 + 20 } );


// geometric series
a = Array.geom( 10, 1, 2 );
a = Array.geom( 10, 1, 1.1 );


// harmonic series:
a = Array.series( 10, 1, 2 );

// methods on arrays:
b = a.reverse;
a.postln; b.postln;

b = a.scramble;
a.postln; b.postln;


b = a.mirror;
a.postln; b.postln;

b = a.mirror1;
a.postln; b.postln;

b = a.mirror2;
a.postln; b.postln;

b = a.stutter(2);
a.postln; b.postln;

b = a.rotate(3);
a.postln; b.postln;

b = a.rotate(-3);
a.postln; b.postln;

a = Array.new(1);
a.add( 2 );
a
a.add( 4 );
a

(
x = [ [1, 2, 3], 6, List["foo", 'bar']];
y = x.lace(12);
x.postln;
y.postln;
)

b = a.slide( 3, 1 );
a.postln; b.postln;

/// iterating over an array:
a.do( { arg item, index; [item,index].postln; });

// iterating and collecting results in a new array:
c = a.collect( { arg item, index; [item,index] });


// math on arrays:
a = Array.series( 10, 1, 2 );
a * 2;

b = Array.series( 10, 1, 1 );
a * b;

a * [1,2];

// ----------- Arrays in SynthDefs --------
When an Array is given as an input to a unit generator it causes an array of
multiple copies of that unit generator to be made, each with a different value
from the input array. This is called multichannel expansion. All but a few
special unit generators perform multichannel expansion. Only Arrays are
expanded, no other type of Collection, not even subclasses of Array.

s.makeGui;
s.boot;

s.meter;

// this create two sine waves, each going to a different channel:
{ SinOsc.ar( 5*[100,200], 0, 0.2 ) }.scope;

a = { SinOsc.ar( 5*[100,200], 0, 0.1 ) }.play;

a = SinOsc.ar( 5*[100,200], 0, 0.2 )

a

{ var a = SinOsc.ar( 5*[100,200], 0, 0.2 ); a.do{ |it,i| it.postln; } }.scope;

a.play;


// if you do not want the signals to go to different channels, you have to downmix them again:

{ Mix.new( SinOsc.ar( 5 * [100,200], 0, 0.2 ) ) }.scope;


// Multichannel expansion will propagate through the expression graph. When a unit generator constructor is called with an array of inputs, it returns an array of instances. If that array is the input to another constructor, then another array is created, and so on.

{ RLPF.ar(Saw.ar(5*[100,250],0.05), XLine.kr(8000,400,5), 0.05) }.play;

// the [100,250] array of frequency inputs to Saw causes Saw.ar to return
// an array of two Saws, that array causes RLPF.ar to create two RLPFs.
// Both RLPFs share a single instance of XLine.

//When a constructor is parameterized by two or more arrays, then the number of channels created is equal to the longest array, with parameters being pulled from each array in parallel. The shorter arrays will wrap. for example, the following:

Pulse.ar([400, 500, 600],[0.5, 0.1], 0.2)

is equivalent to:

[ Pulse.ar(400,0.5,0.2), Pulse.ar(500,0.1,0.2), Pulse.ar(600,0.5,0.2) ]

A more complex example based on the Saw example above is given below. In this
example, the XLine is expanded to two instances, one going from 8000 Hz to 400
Hz and the other going in the opposite direction from 500 Hz to 7000 Hz. These
two XLines are 'married' to the two Saw oscillators and used to parameterize
two copies of RLPF. So on the left channel a 100 Hz Saw is filtered from 8000
Hz to 400 Hz and on the right channel a 250 Hz Saw is filtered from 500 Hz to
7000 Hz.

{ RLPF.ar(Saw.ar([100,250],0.05), XLine.kr([8000,500],[400,7000],5), 0.05) }.play;

// see also helpfile [Multichannel]

// another example:

a = Array.series( 10, 1, 2 );
b = Array.geom( 10, 1, 0.99 );
c = Array.geom( 10, 0.05, 0.99 );


{ Mix.new(SinOsc.ar( a * 100, 0, b )) * 0.1 }.play;

a
b

{ Mix.new(SinOsc.ar( a * 100, 0, b ) * EnvGen.kr( Env.perc(0.05,0.95), 1, timeScale: b ) ) * 0.1 }.play;

10.do{ |i|
	{ Mix.new(SinOsc.ar( a[i]*100,0,0.1 ) * EnvGen.kr( Env.perc(0.05*i+0.05,0.95), 1, timeScale: b[i] ) ) * 0.1 }.play;
}

